/**
 * Copyright (c) 2005 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda2/).
 */
package org.xmloperator.lambda2.net.impl;

import org.xmloperator.lambda2.net.model.Cell;
import org.xmloperator.lambda2.net.model.Port;
import org.xmloperator.lambda2.term.impl.VariableImpl;

/**
 * Port implementation. In the mapping with terms, a Port is a Variable.
 */
public class PortImpl extends VariableImpl implements Port {
	CellImpl cell;
	int portNumber;
	PortImpl connectedPort = null;

	/**
	 * Free port constructor.
	 */
	public PortImpl() {
		this.cell = null;
		this.portNumber = -1;
	}

	/**
	 * Cell port constructor.
	 * 
	 * @param cell
	 *            a Cell.
	 */
	public PortImpl(CellImpl cell, int portNumber) {
		if (cell == null || portNumber < 0)
			throw new IllegalArgumentException();
		this.cell = cell;
		this.portNumber = portNumber;
	}

	//////////////////////
	// Port implementation
	//////////////////////

	public final Cell getCell() {
		return this.cell;
	}

	public final boolean isFree() {
		return this.cell == null;
	}

	public final int getPortNumber() {
		return this.portNumber;
	}

	public final boolean isPrincipal() {
		return this.portNumber == 0;
	}

	public final Port getConnectedPort() {
		return this.connectedPort;
	}

	public final boolean isConnected() {
		return this.connectedPort != null;
	}

	public final void connectTo(Port port) {
		if (port == this)
			throw new IllegalArgumentException();
		if (this.connectedPort != null)
			this.connectedPort.connectedPort = null;
		PortImpl otherPort = (PortImpl) port;
		if (otherPort.connectedPort != null)
			otherPort.connectedPort.connectedPort = null;
		this.connectedPort = otherPort;
		otherPort.connectedPort = this;
	}

	public final void disconnect() {
		if (this.connectedPort != null) {
			this.connectedPort.connectedPort = null;
			this.connectedPort = null;
		}
	}
}
