/**
 * Copyright (c) 2005 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda2/).
 */
package org.xmloperator.lambda2.net.impl;

import org.xmloperator.lambda2.net.model.Cell;
import org.xmloperator.lambda2.net.model.Port;
import org.xmloperator.lambda2.net.model.Stack;
import org.xmloperator.lambda2.net.model.WireCell;

/**
 * Wire cell implementation.
 */
public final class WireCellImpl extends CellImpl implements WireCell {
	final PortImpl port1 = new PortImpl(this, 1);

	public WireCellImpl() {
	}

	//////////////////////
	// Cell implementation
	//////////////////////

	public final int getSymbol() {
		return SYMBOL_WIRE;
	}

	public int getSymbolArity() {
		return 1;
	}
	
	public final boolean isWireCell() {
		return true;
	}

	public Port getPort(int portNumber) {
		switch (portNumber) {
			case 0 :
				return this;
			case 1 :
				return this.port1;
			default :
				throw new IllegalArgumentException();
		}
	}

	public void collectPorts(Stack portStack) {
		super.collectPorts(portStack);
		portStack.push(this.port1);
	}

	public Cell cloneCell() {
		return new WireCellImpl();
	}

	public final int interact() {
		PortImpl connectedPort0 = this.connectedPort;
		PortImpl connectedPort1 = this.port1.connectedPort;
		if (connectedPort1 != connectedPort0)
			connectedPort1.connectTo(connectedPort0);
		return INTERACTION_WIRE;
	}

	//////////////////////////
	// WireCell implementation
	//////////////////////////

	public final Port getAuxiliaryPort() {
		return this.port1;
	}
}
