/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/inm/).
 */
package org.xmloperator.lambda2.net.model;

/**
 * Cell used for decoding a LambdaNet that represents an abstraction.
 * <p>
 * The Cell has an index in order to represent a variable.
 * </p>
 * <p>
 * The Decoder increments its arity by interacting with an applicator Cell.
 * </p>
 */
public interface Decoder extends Cell {

	/**
	 * Returns the index attached to this Decoder.
	 * 
	 * @return the index attached to this Decoder.
	 */
	public int getIndex();

	/**
	 * Adds a new auxiliary Port to this Cell and returns it. The symbol arity
	 * is incremented. The new Port has the new arity as port number.
	 * 
	 * @return the created auxiliary Port.
	 */
	public Port addNewAuxiliaryPort();

	/**
	 * Returns the sequence count of this unary Decoder.
	 * <p>
	 * This doesn't make sense for a Decoder that is not unary.
	 * </p>
	 * 
	 * @return the sequence count of this unary Decoder.
	 */
	public long getSequenceCount();

	/**
	 * Adds a given sequence count to this unary Decoder.
	 * <p>
	 * This doesn't make sense for a Decoder that is not unary.
	 * </p>
	 * 
	 * @param sequenceCount
	 *            a sequence count.
	 * @throws IllegalArgumentException
	 *             if the given sequence count is <= 0.
	 * @throws IllegalStateException
	 *             if this Decoder is not unary.
	 */
	public void addSequenceCount(long sequenceCount);
}
