/**
 * Copyright (c) 2005 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda2/).
 */
package org.xmloperator.lambda2.net.model;

/**
 * Abstraction multiplexor or application demultiplexor.
 */
public interface PentaryMultiplexor extends Cell {

	/**
	 * Returns true if this cell is an abstraction multiplexor, false if it is
	 * an application de-multiplexor.
	 * 
	 * @return true if this cell is an abstraction multiplexor, false if it is
	 *         an application de-multiplexor.
	 */
	public boolean isAbstractionMultiplexor();

	/**
	 * Returns the body Port of this (de-)multiplexor.
	 * 
	 * @return the body Port of this (de-)multiplexor.
	 */
	public Port getBodyPort();

	/**
	 * Returns the variable Port of this (de-)multiplexor.
	 * 
	 * @return the variable Port of this (de-)multiplexor.
	 */
	public Port getVariablePort();

	/**
	 * Returns the Port of this (de-)multiplexor that maps to the Port 0 of a
	 * duplicator.
	 * 
	 * @return the Port of this (de-)multiplexor that maps to the Port 0 of a
	 *         duplicator.
	 */
	public Port getDuplicatorPort0();

	/**
	 * Returns the Port of this (de-)multiplexor that maps to the Port 1 of a
	 * duplicator.
	 * 
	 * @return the Port of this (de-)multiplexor that maps to the Port 1 of a
	 *         duplicator.
	 */
	public Port getDuplicatorPort1();

	/**
	 * Returns the Port of this (de-)multiplexor that maps to the Port 2 of a
	 * duplicator.
	 * 
	 * @return the Port of this (de-)multiplexor that maps to the Port 2 of a
	 *         duplicator.
	 */
	public Port getDuplicatorPort2();
}
