/**
 * Copyright (c) 2005 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda2/).
 */
package org.xmloperator.lambda2.term.impl;

import java.util.Map;
import java.util.Set;

import org.xmloperator.lambda2.term.model.Abstraction;
import org.xmloperator.lambda2.term.model.Term;
import org.xmloperator.lambda2.term.model.Variable;

/**
 * Zero or more variable Abstraction implementation.
 */
public class Abstraction0Impl extends TermImpl implements Abstraction {
	protected static final boolean ASSERTION = Test.ASSERTION;
	final TermImpl body;

	public Abstraction0Impl(TermImpl body) {
		if (body == null)
			throw new IllegalArgumentException();
		this.body = body;
	}

	void addFreeVariables(Set boundVariables, Set freeVariables) {
		this.body.addFreeVariables(boundVariables, freeVariables);
	}

	void toText(StringBuffer buffer, int boundVariableStackSize,
			Map fromBoundVariableToName) {
		String variableName = "x" + Integer.toString(++boundVariableStackSize);
		buffer.append('\\');
		buffer.append(variableName);
		buffer.append('.');
		this.body.toText(buffer, boundVariableStackSize,
				fromBoundVariableToName);
	}

	// ////////////////////
	// Term implementation
	// ////////////////////

	public final int getTermType() {
		return TYPE_ABSTRACTION;
	}

	public final boolean isAbstraction() {
		return true;
	}

	// ///////////////////////////
	// Abstraction implementation
	// ///////////////////////////

	public final Term getBody() {
		return this.body;
	}

	public int getBoundVariableCount() {
		return 0;
	}

	public Variable getBoundVariable(int index) {
		throw new IllegalArgumentException();
	}
}