/**
 * Copyright (c) 2005 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda2/).
 */
package org.xmloperator.lambda2.term.impl;

import java.util.Map;
import java.util.Set;

import org.xmloperator.lambda2.term.model.Variable;

/**
 * Two or more variables Abstraction implementation.
 */
public final class AbstractionNImpl extends Abstraction0Impl {
	final Variable[] boundVariables;

	public AbstractionNImpl(TermImpl body, Variable[] boundVariables) {
		super(body);
		if (boundVariables == null)
			throw new IllegalArgumentException();
		this.boundVariables = boundVariables;
	}

	final void addFreeVariables(Set boundVariables, Set freeVariables) {
		int boundVariableCount = this.boundVariables.length;
		for (int k = 0; k < boundVariableCount; k++)
			if (!boundVariables.add(this.boundVariables[k]))
				throw new IllegalStateException(
						"Two abstractions for the same bound variable");
		this.body.addFreeVariables(boundVariables, freeVariables);
		if (ASSERTION)
			for (int k = 0; k < boundVariableCount; k++)
				if (boundVariables.contains(this.boundVariables[k]))
					throw new IllegalStateException(
							"Bound variable unused inside its abstraction");
	}

	final void toText(StringBuffer buffer, int boundVariableStackSize,
			Map fromBoundVariableToName) {
		String variableName = "x" + Integer.toString(++boundVariableStackSize);
		int boundVariableCount = this.boundVariables.length;
		for (int k = 0; k < boundVariableCount; k++)
			if (fromBoundVariableToName.put(this.boundVariables[k],
					variableName) != null)
				throw new IllegalStateException(
						"Two abstractions for the same bound variable");
		buffer.append('\\');
		buffer.append(variableName);
		buffer.append('.');
		this.body.toText(buffer, boundVariableStackSize,
				fromBoundVariableToName);
		if (ASSERTION)
			for (int k = 0; k < boundVariableCount; k++)
				if (fromBoundVariableToName.containsKey(this.boundVariables[k]))
					throw new IllegalStateException(
							"Bound variable unused inside its abstraction");
	}

	// ///////////////////////////
	// Abstraction implementation
	// ///////////////////////////

	public int getBoundVariableCount() {
		return this.boundVariables.length;
	}

	public Variable getBoundVariable(int index) {
		return this.boundVariables[index];
	}
}