/**
 * Copyright (c) 2005 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda2/).
 */
package org.xmloperator.lambda2.term.impl;

import java.util.Map;
import java.util.Set;

import org.xmloperator.lambda2.term.model.Application;
import org.xmloperator.lambda2.term.model.Term;

/**
 * Application implementation.
 */
public class Application1Impl extends TermImpl implements Application {
	final TermImpl function;
	final TermImpl argument;

	public Application1Impl(TermImpl function, TermImpl argument) {
		if (function == null || argument == null)
			throw new IllegalArgumentException();
		this.function = function;
		this.argument = argument;
	}

	final void addFreeVariables(Set boundVariables, Set freeVariables) {
		this.function.addFreeVariables(boundVariables, freeVariables);
		this.argument.addFreeVariables(boundVariables, freeVariables);
	}

	final void toText(StringBuffer buffer, int boundVariableStackSize,
			Map fromBoundVariableToName) {
		boolean mustFunctionBeParenthesed = this.function.isAbstraction()
				|| (this.function.isApplication() && ((Application) this.function)
						.getArgument().isAbstraction());
		boolean mustArgumentBeParenthesed = this.argument.isApplication();
		if (mustFunctionBeParenthesed)
			buffer.append('(');
		this.function
				.toText(buffer, boundVariableStackSize, fromBoundVariableToName);
		if (mustFunctionBeParenthesed)
			buffer.append(')');
		this.exponentToText(buffer);
		if (mustArgumentBeParenthesed)
			buffer.append('(');
		this.argument
				.toText(buffer, boundVariableStackSize, fromBoundVariableToName);
		if (mustArgumentBeParenthesed)
			buffer.append(')');
	}

	protected void exponentToText(StringBuffer buffer) {
	}
	
	// ////////////////////
	// Term implementation
	// ////////////////////

	public final int getTermType() {
		return TYPE_APPLICATION;
	}

	public final boolean isApplication() {
		return true;
	}

	// ///////////////////////////
	// Application implementation
	// ///////////////////////////

	public final Term getFunction() {
		return this.function;
	}

	public final Term getArgument() {
		return this.argument;
	}

	public long getFunctionExponent() {
		return 1L;
	}
}
